/*
 *  Blinkless Demo -- WiFiBoy32 Demo Code for Arduino/ESP32
 *   
 *  Nov 26, 2017 Created. (derek@wifiboy.org)
 *
 */
#include <wifiboy32.h>
#include "wb32_sprite.h"

void blit_str256(const char *str, int x, int y)
{
  for(int i=0; i<strlen(str); i++) {
    if (str[i]>='@'&& str[i]<=']') 
      wb32_blitBuf8(8*(str[i]-'@'),0,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
    if (str[i]>='!'&& str[i]<='>') 
      wb32_blitBuf8(8*(str[i]-'!'),8,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
    if (str[i]=='?') wb32_blitBuf8(8*14,16,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
    if (str[i]=='c') wb32_blitBuf8(8*13,16,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
    if (str[i]=='w') wb32_blitBuf8(7,16,240, x+i*8, y, 26, 8, (uint8_t *)sprites);
    if (str[i]=='x') wb32_blitBuf8(42,16,240, x+i*8, y, 61, 8, (uint8_t *)sprites);
  }
}

void blit_num256(uint16_t num, uint16_t x, uint16_t y, uint8_t color_mode)
{
  uint16_t  d[5]; // five digit number
    
  d[0]=num/10000;
  d[1]=(num-d[0]*10000)/1000;
  d[2]=(num-d[0]*10000-d[1]*1000)/100;
  d[3]=(num-d[0]*10000-d[1]*1000-d[2]*100)/10;
  d[4]=num-d[0]*10000-d[1]*1000-d[2]*100-d[3]/10;
    
  for(int i=0; i<5; i++) {
    wb32_blitBuf8(d[i]*8+120, color_mode*8, 240, x+i*8, y, 8, 8, (uint8_t *)sprites);
  }
}

int ball_x, ball_y, dx, dy, bar_x, score;

void setup() {

  wb32_init(); // init WiFiBoy32 engine

  wb32_initBuf8(); // prepare 76800-byte off-screen buffer
  
  for(int i=0; i<256; i++) // setup palette (8bit = 256 colors)
    wb32_setPal8(i, wb32_color565(sprite_pal[i][0],sprite_pal[i][1],sprite_pal[i][2]));
  
  ball_x = ball_y = 10;
  dx = dy = 5;
  score = 0;
  
  bar_x = 100;
  pinMode(17, INPUT); 
  pinMode(32, INPUT);

}

void loop() 
{
  wb32_clearBuf8(); // clear off-screen buffer

  if (digitalRead(17)==0) bar_x -= 8;
  if (bar_x < 0) bar_x = 0;
  if (digitalRead(32)==0) bar_x += 8;
  if (bar_x > 200) bar_x = 200;
  
  wb32_blitBuf8(68, 218, 240, bar_x, 310, 40, 8, (uint8_t *)sprites);
  
  ball_x += dx;
  if ((ball_x > 230) || (ball_x < 0)) { dx=-dx; ball_x += dx; }
  ball_y += dy;
  if (ball_y < 0) { dy=-dy; ball_y += dy; }
  
  if (ball_y > 300) {
    if ((ball_x+10 >= bar_x) && (ball_x <= bar_x+40)) {
      dy=-dy;
      score++;
    }
    else {
      ball_x = ball_y = 10; // restart
      dx = dy = 5;
      score = 0;
    }
  }
  
  //wb32_blitBuf8(42, 84, 240, ball_x, ball_y, 12, 12, (uint8_t *)sprites);
  wb32_blitBuf8(192, 192, 240, ball_x, ball_y, 10, 10, (uint8_t *)sprites);

  blit_str256("PONG DEMO", 80, 10);
  blit_str256("SCORE", 100, 80);
  blit_num256(score*100, 100, 100, 4);
  
  
  wb32_blit8(); // blit to LCD
  //delay(50);
}
