/*
 *  Chess-L Game Demo -- WiFiBoy32 Demo Code for Arduino/ESP32
 *  
 *  Chess-L was invented by Dr. Edward de Bono.
 *  
 *  This game is designed to be INVINCIBLE. Please try hard to beat it!
 *  
 *  Nov 21, 2017 Created. (derek@wifiboy.org & ricky@wifiboy.org)
 *  (based on chess learning theory of Derek Lai, Yi-hung Kuo, Lian-Chang Wang in 1985.)
 *  
 *  Copyright 2017 WiFiBoy.org, All rights reserved.
 *
 */
#include "wifiboy32.h"

int shapes[8][4]={
  {0,-4,-8,-9}, {0,-4,-8,-7}, {0,-1,-2,-6}, {0,-1,-2, 2},
  {0,1,2,-2},{0,1,2,6},{0,4,8,7},{0,4,8,9}
};
int shx[8][3]={
  {0,0,-1},{0,0,1},{-1,-2,-2},{-1,-2,-2},
  {1,2,2},{1,2,2},{0,0,-1},{0,0,1}
};
int shy[8][3]={
  {-1,-2,-2},{-1,-2,-2},{0,0,-1},{0,0,1},
  {0,0,-1},{0,0,1},{1,2,2},{1,2,2}
};
uint8_t mates[696]={
  0xe8,0x4a,0x2f,0xe8,0x29,0x4b,0xe8,0xb5,0x14,0xe8,0x97,0x14,0x60,0x8e,0x27,
  0xe8,0x4a,0x3f,0xe8,0x29,0x4f,0xe8,0xb5,0x24,0xe8,0x97,0x24,0x60,0x97,0x28,
  0xe8,0x4a,0x7f,0xe8,0x42,0xaf,0xe8,0x1a,0x4b,0xe8,0x2b,0x49,0xf9,0x2b,0x58
};
uint8_t mates_more[]={
  0xe8,0x1a,0x4f,0xe8,0x1a,0x7b,0x60,0xac,0x27,0xe8,0x1a,0x0b,0xe8,0x1a,0x2b,
  0xf9,0x4a,0x8b,0xe8,0x1a,0x0f,0xe8,0x1a,0x2f,0xe8,0x4a,0xbf,0xe8,0x1a,0x7f,
  0xe8,0x1a,0xbf,0xe8,0x1a,0x27,0xf9,0x68,0x2a,0xf9,0x68,0x3a
};
uint8_t rotab[16]={0xc,0x8,0x4,0x0,0xd,0x9,0x5,0x1,0xe,0xa,0x6,0x2,0xf,0xb,0x7,0x3};
uint8_t mitab[16]={0x3,0x2,0x1,0x0,0x7,0x6,0x5,0x4,0xb,0xa,0x9,0x8,0xf,0xe,0xd,0xc};

void draw_board()
{
  wb32_fillRect(10,50,222,222,0);
  for(int i=0; i<5; i++) {
    wb32_drawFastHLine(10,i*55+50, 220, 0xffff);
    wb32_drawFastVLine(10+i*55,50, 220, 0xffff);
  }
}

uint16_t colors[]={0x00f8, 0x1f00, 0xe007};

void draw_l(int m, int n, int side)
{
  int i;
  for(i=0; i<8; i++) if ((n-m)==shapes[i][3]) break;
  for(int j=0; j<4; j++) 
    wb32_fillRect((shapes[i][j]+m)%4*55+10+7, (shapes[i][j]+m)/4*55+50+7, 
      42, 42, colors[side]);
}

void draw_n(uint8_t n)
{
  int n1 = n/16;
  int n2 = n%16;
  wb32_fillRect(n1%4*55+10+7, n1/4*55+50+7, 42, 42, colors[2]);
  wb32_fillRect(n2%4*55+10+7, n2/4*55+50+7, 42, 42, colors[2]);
}

void show_board(uint8_t a, uint8_t b, uint8_t n)
{
  draw_board();
  draw_l(a/16, a%16, 0);
  draw_l(b/16, b%16, 1);
  draw_n(n);
}

int slist[100];

int check_a(int a, int b, int n)
{
  int board[16], i, j, k, ix, iy, px, py, c, d, s, count;
  
  for(i=0; i<16; i++) board[i]=0;
  c=b/16; d=b%16;
  for(i=0; i<8; i++) if ((d-c)==shapes[i][3]) break;
  for(int j=0; j<4; j++) board[shapes[i][j]+c]=1;
  board[n/16]=1;
  board[n%16]=1;
  count=0;
  for(i=0; i<16; i++){
    if (board[i]>0) continue;
    ix = i%4;
    iy = i/4; 
    for(j=0; j<8; j++) {
      for(k=0; k<3; k++) {
        s=((i*16)+(shapes[j][3]+i));
        px = shx[j][k];
        py = shy[j][k];
        if (((ix+px)>3)||((ix+px)<0)) break;
        if (((iy+py)>3)||((iy+py)<0)) break;
        if (board[shapes[j][k+1]+i]>0) break;
        if (k==2) 
          if (s!=a) slist[count++]=s;
      }
    }
  }
  return count;
}

void init_matelist()
{
  for(int i=0; i<15; i++) {
    for(int j=0; j<3; j++) {
      int c=mates[i*3+j]/16; 
      int d=mates[i*3+j]%16;
      mates[i*3+45+j]=rotab[c]*16+rotab[d];
      mates[i*3+90+j]=rotab[rotab[c]]*16+rotab[rotab[d]];
      mates[i*3+135+j]=rotab[rotab[rotab[c]]]*16+rotab[rotab[rotab[d]]];
    }
  }
  for(int i=0; i<15*3*4; i++) {
    int c=mates[i]/16; 
    int d=mates[i]%16;
    mates[i+180]=mitab[c]*16+mitab[d];
  }
  
  for(int i=0; i<15*3*4*2; i+=3) {
    if (i%15==0) Serial.println(" ");
    Serial.printf("(%2X %2X %2X) ", mates[i], mates[i+1], mates[i+2]);
  }
  Serial.println("< More >");
  
  for(int i=0; i<14; i++) {
    mates[i*3+360]=mates_more[i*3];
    mates[i*3+361]=mates_more[i*3+1];
    mates[i*3+362]=mates_more[i*3+2];
    for(int j=0; j<3; j++) {
      int c=mates_more[i*3+j]/16; 
      int d=mates_more[i*3+j]%16;
      mates[i*3+42+j+360]=rotab[c]*16+rotab[d];
      mates[i*3+84+j+360]=rotab[rotab[c]]*16+rotab[rotab[d]];
      mates[i*3+126+j+360]=rotab[rotab[rotab[c]]]*16+rotab[rotab[rotab[d]]];
    }
  }
  for(int i=0; i<14*3*4; i++) {
    int c=mates[i+360]/16; 
    int d=mates[i+360]%16;
    mates[i+360+168]=mitab[c]*16+mitab[d];
  }
  for(int i=0; i<14*3*4*2; i+=3) {
    if (i%15==0) Serial.println(" ");
    Serial.printf("(%2X %2X %2X) ", mates[i+360], mates[i+361], mates[i+362]);
  }
}

int check_mate(int a, int b, int n)
{
  int cc, c, d, e, f;
  
  cc=check_a(b,a,n);
  Serial.printf("\nVerify:(%02X %02X %02X) ", a,b,n);
  for(int i=0; i<29*8; i++) {
    if (a==mates[i*3]){
      Serial.printf(":(%02X %02X %02X) ", a,b,n);
      for(int j=0; j<cc; j++) {
        if (mates[i*3+1]==slist[j]) {
          Serial.printf("\nVerify:(%02X %02X %02X) ", a,slist[j],n);
          c=n/16;
          d=n%16;
          e=mates[i*3+2]/16;
          f=mates[i*3+2]%16;
          if ((c==e)||(c==f)||(d==e)||(d==f)) {
            Serial.printf("\nGot:(%02X %02X %02X) ", a,slist[j],n);
            return i;
          }
        }
      }
    }
  }
  return -1;
}


int board_a, board_b, board_n;

void computer_play()
{
  int x, y, c, d, i, cc, n, board[16], xlist[100];
  
  cc=check_a(board_b, board_a, board_n);
  if (cc>0) {
    if ((x=check_mate(board_a, board_b, board_n))>=0) {
      board_b=mates[x*3+1];
      board_n=mates[x*3+2];  
      wb32_drawString("!", 0, 290, 2, 2);  
    } else {
      for(i=0; i<cc; i++) xlist[i]=slist[i];
      x = random(0,cc);
      n = board_n;
      while(1) {
        board_b=xlist[x];
        board_n=n;
        for(i=0; i<16; i++) board[i]=0;
        c=board_a/16; d=board_a%16;
        for(i=0; i<8; i++) if ((d-c)==shapes[i][3]) break;
        for(int j=0; j<4; j++) board[shapes[i][j]+c]=1;
        c=board_b/16; d=board_b%16;
        for(i=0; i<8; i++) if ((d-c)==shapes[i][3]) break;
        for(int j=0; j<4; j++) board[shapes[i][j]+c]=1;
        c=board_n/16;
        d=board_n%16;
        if (random(0,2)==0) { board[c]=1; d=c; } else board[d]=1;
        y=random(0,16);
        for(i=0; i<16; i++) {
          if (board[y]==0) break;
          y=y+1; if (y>15) y=0;
        }
        if (check_mate(board_b, board_a, y*16+d)<0) {
          board_n = y*16+d;
          break;
        }
        x = x + 1; 
        if (x>cc-1) x = 0;        
      }
    }
  } else {
    wb32_fillRect(0,0,240,45,0);
    wb32_drawString("You Win!", 5, 0, 2, 2);
    wb32_drawString("Press Start to RePlay", 5, 20, 2, 2);
    //check_mate(board_a, board_b, board_n);
    board_a=0xa1;
    board_b=0x5e;
    board_n=0x0f;
    while(digitalRead(23)==1);
  }
}

void setup() 
{
  Serial.begin(115200);
  wb32_init();
  wb32_setTextColor(wbGREEN, wbGREEN);
  wb32_drawString("by WiFiBoy.org", 120, 290, 1, 2);
  wb32_setTextColor(0xffff, 0xffff);
  wb32_drawString("Chess-L", 25, 290, 2, 2);
  pinMode(17,INPUT); pinMode(32,INPUT); pinMode(34,INPUT); pinMode(23,INPUT);
  board_a=0xa1;
  board_b=0x5e;
  board_n=0x0f;
  init_matelist();
}

void loop() 
{
  int selecting;
  int lastkey;
  int cc, c, d, n1, i;
  int board[16];

  show_board(board_a, board_b, board_n);
  cc=check_a(board_a, board_b, board_n);
  if (cc==0) {
    wb32_fillRect(0,0,240,45,0);
    wb32_drawString("You Lose!", 5, 0, 2, 2);
    wb32_drawString("Press Start to RePlay", 5, 23, 2, 2);
    board_a=0xa1;
    board_b=0x5e;
    board_n=0x0f;
    while(digitalRead(23)==1);
    wb32_fillRect(0,290,5,15,0);
    return;
  }
  wb32_fillRect(0,0,240,45,0);
  wb32_drawString("Move Your Red L (L/R)", 5, 0, 2, 2);
  wb32_drawString("Press B to Confirm", 5, 23, 2, 2);
  selecting=1;
  lastkey=0;
  c=0;
  while(selecting>0) {
    if ((digitalRead(32)==0) && (lastkey==0)){
      lastkey=1;
      c++; if (c>cc-1) c=0;
      board_a=slist[c];
      show_board(board_a, board_b, board_n);
      selecting=2;
    }
    if ((digitalRead(32)==1) && (lastkey==1)) lastkey=0;
    if ((digitalRead(17)==0) && (lastkey==0)){
      lastkey=3;
      c--; if (c<0) c=cc-1;
      board_a=slist[c];
      show_board(board_a, board_b, board_n);
      selecting=2;
    }
    if ((digitalRead(17)==1) && (lastkey==3)) lastkey=0;
    if ((digitalRead(34)==0) && (lastkey==0)) lastkey=2;
    if ((digitalRead(34)==1) && (lastkey==2)) {
      lastkey=0;
      if (selecting==2) selecting=0;
    }
  }

  wb32_fillRect(0,0,240,45,0);
  wb32_drawString("Select N (L/R)", 5, 0, 2, 2);
  wb32_drawString("Confirm (B)", 5, 23, 2, 2);
  // select n
  selecting=1;
  c=0;
  n1 = board_n / 16;
  wb32_fillRect(n1%4*55+10+13, n1/4*55+50+13, 30, 30, 0xffff);
  while(selecting==1) {
    if ((digitalRead(32)==0) && (lastkey==0)){
      lastkey=1;
      c=1-c;
      show_board(board_a, board_b, board_n);
      if (c) n1 = board_n % 16;
      else n1 = board_n / 16;
      wb32_fillRect(n1%4*55+10+13, n1/4*55+50+13, 30, 30, 0xffff);
    }
    if ((digitalRead(32)==1) && (lastkey==1)) lastkey=0;
    if ((digitalRead(17)==0) && (lastkey==0)){
      lastkey=3;
      c=1-c;
      show_board(board_a, board_b, board_n);
      if (c) n1 = board_n % 16;
      else n1 = board_n / 16;
      wb32_fillRect(n1%4*55+10+13, n1/4*55+50+13, 30, 30, 0xffff);
    }
    if ((digitalRead(17)==1) && (lastkey==3)) lastkey=0;
    if ((digitalRead(34)==0) && (lastkey==0)) lastkey=2;
    if ((digitalRead(34)==1) && (lastkey==2)) {
      lastkey=0;
      selecting=0;
    }
  }

  wb32_fillRect(0,0,240,45,0);
  wb32_drawString("Place N (L/R)", 5, 0, 2, 2);
  wb32_drawString("Confirm (B)", 5, 23, 2, 2);
  // place n
  if (c==0) n1=board_n % 16;
  else n1 = board_n / 16;
  show_board(board_a, board_b, n1*16+n1);
  for(i=0; i<16; i++) board[i]=0;
  c=board_a/16; d=board_a%16;
  for(i=0; i<8; i++) if ((d-c)==shapes[i][3]) break;
  for(int j=0; j<4; j++) board[shapes[i][j]+c]=1;
  c=board_b/16; d=board_b%16;
  for(i=0; i<8; i++) if ((d-c)==shapes[i][3]) break;
  for(int j=0; j<4; j++) board[shapes[i][j]+c]=1;
  board[n1]=1;
  c=0;
  while(board[c]==1) { c++; if (c>15) c=0; }
  show_board(board_a, board_b, n1*16+n1);
  wb32_fillRect((c%4)*55+10+13, (c/4)*55+50+13, 30, 30, 0xffff);
  selecting=1;
  while(selecting==1) {
    if ((digitalRead(32)==0) && (lastkey==0)){
      lastkey=1;
      c++; if (c>15) c=0;
      while(board[c]==1) { c++; if (c>15) c=0; }
      show_board(board_a, board_b, n1*16+n1);
      wb32_fillRect((c%4)*55+10+13, (c/4)*55+50+13, 30, 30, 0xffff);
    }
    if ((digitalRead(32)==1) && (lastkey==1)) lastkey=0;
    if ((digitalRead(17)==0) && (lastkey==0)){
      lastkey=3;
      c--; if (c<0) c=15;
      while(board[c]==1) { c--; if (c<0) c=15; }
      show_board(board_a, board_b, n1*16+n1);
      wb32_fillRect((c%4)*55+10+13, (c/4)*55+50+13, 30, 30, 0xffff);
    }
    if ((digitalRead(17)==1) && (lastkey==3)) lastkey=0;
    if ((digitalRead(34)==0) && (lastkey==0)) lastkey=2;
    if ((digitalRead(34)==1) && (lastkey==2)) {
      lastkey=0;
      selecting=0;
      if (c<n1) board_n=c*16+n1; else board_n=n1*16+c;
      show_board(board_a, board_b, board_n);
    }
  }

  computer_play();
  
}
