/*
 *  Pong Test 2 -- WiFiBoy32 Demo Code for Arduino/ESP32
 *
 *  Nov 25, 2017 Created. (derek@wifiboy.org)
 *
 */
#include "wifiboy32.h"
#include "WiFi.h"

WiFiServer server(1234);  // port 1234 for game server
WiFiClient client;        

int ball_x, ball_y, dx, dy, bar_x, bar_x2, game_mode;
uint8_t buf[10];

void setup_sound()
{
  ledcSetup(1, 400, 8); // ledc/pwm ch#1, freq=400, duty resolution=8bit (0-255)
  ledcAttachPin(25, 1); // IO#25=buzzer, attached to pwm#1
}

void make_sound(int volume)
{
  ledcWrite(1, volume);
}

void sound_freq(int freq)
{
  ledcSetup(1, freq, 8);
}

void setup() 
{
  wb32_init();
  wb32_setTextColor(wbCYAN, wbCYAN);
  
  ball_x = ball_y = 10;
  dx = dy = 3;
  bar_x = 100;
  pinMode(17, INPUT); 
  pinMode(32, INPUT);
  setup_sound();

  game_mode=0;
  if (digitalRead(32)==0) { 
    wb32_drawString("PLAYER-1", 75, 120, 2, 2);
    WiFi.softAP("Pong1", "12345678");
    server.begin();
    game_mode=1; 
    while(1) {
      client = server.available();
      if (client) break;
    }
  } else if (digitalRead(17)==0) { 
    wb32_drawString("PLAYER-2", 75, 120, 2, 2);
    WiFi.begin("Pong1", "12345678");
    while(WiFi.status()!=WL_CONNECTED) delay(500);
    client.connect("192.168.4.1", 1234);
    while(!client.connected());
    game_mode=2; 
  }
  if (game_mode > 0) bar_x2 = 100;
  wb32_fillScreen(0);
}

void loop() 
{
  if (digitalRead(17)==0) bar_x -= 4;
  if (bar_x < 0) bar_x = 0;
  if (digitalRead(32)==0) bar_x += 4;
  if (bar_x > 200) bar_x = 200;
  
  wb32_fillRect(bar_x, 310, 40, 8, wbRED);
  
  if (game_mode>0) {
    while (client.available()) {
      client.read(buf, 6);
      if (buf[0]=='x') { 
        bar_x2 = (buf[2]-48)*100+(buf[3]-48)*10+buf[4]-48;
      }
    }
    wb32_fillRect(bar_x2, 0, 40, 8, wbRED);
    sprintf((char*)buf, "x=%03d", bar_x);
    client.print((char *)buf);
  } 
  
  ball_x += dx;
  if ((ball_x > 230) || (ball_x < 0)) { 
    dx=-dx; ball_x += dx; 
    make_sound(50);;
  }
  ball_y += dy;
  if (ball_y < 8) { 
    dy=-dy; ball_y += dy; 
    make_sound(50); 
  }
  
  if (ball_y > 300) {
    dy=-dy;
 /*   
    if ((ball_x+10 >= bar_x) && (ball_x <= bar_x+40)) {
      dy=-dy;
      make_sound(50); 
    } else {
      ball_x = ball_y = 10; // restart
      dx = dy = 3;
      sound_freq(100);
      make_sound(50);
      //delay(500);
      sound_freq(400);
    }
    */
  }
  
  wb32_fillRect(ball_x, ball_y, 10, 10, wbGREEN);

  delay(30);
  
  wb32_fillRect(ball_x, ball_y, 10, 10, wbBLACK);
  wb32_fillRect(bar_x, 310, 40, 8, wbBLACK);
  if (game_mode>0) wb32_fillRect(bar_x2, 0, 40, 8, wbBLACK);
  ledcWrite(1, 0);
}
